@@ -63,8 +63,11 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-#if defined(HAVE_CONFIG_H)
-#include <config_ac.h>
+#ifndef PIXMAN_REGION_MAX
+#error "This file should be #included from pixman-region16.c, not compiled directly"
+#endif
+#ifndef CONFIG_AC_H
+#error "config_ac.h not visible in pixman-region.c"
 #endif
 
 #define PIXREGION_NIL(reg) ((reg)->data && !(reg)->data->numRects)
@@ -2652,7 +2655,7 @@ PREFIX (_selfcheck) (region_type_t *reg)
     }
     else
     {
-        box_type_t *pbox_p, * pbox_n;
+        box_type_t *pbox_p, *pbox_n;
         box_type_t box;
 
         pbox_p = PIXREGION_RECTS (reg);
