@@ -32,6 +32,10 @@ setxkbmap -model pc105 -layout cz
 setxkbmap -model pc104 -layout de
 ./xrdp-genkeymap ../instfiles/km-00000407.ini
 
+# Hungarian 'hu' 0x0000040e
+setxkbmap -model pc105 -layout hu
+./xrdp-genkeymap ../instfiles/km-0000040e.ini
+
 # Italian 'it' 0x00000410
 setxkbmap -model pc104 -layout it
 ./xrdp-genkeymap ../instfiles/km-00000410.ini
@@ -52,6 +56,15 @@ setxkbmap -model pc104 -layout ru
 setxkbmap -model pc104 -layout se
 ./xrdp-genkeymap ../instfiles/km-0000041d.ini
 
+# Latvian 'lv' 0x00000426
+setxkbmap -model pc104 -layout lv -variant ergonomic
+./xrdp-genkeymap ../instfiles/km-00000426.ini
+
+# Latvian standard 'lv' 0x00010426
+# 20426 Latvian(Standard) is symlinked to 10426 Latvian(QWERTY) elsewhere
+setxkbmap -model pc104 -layout lv
+./xrdp-genkeymap ../instfiles/km-00010426.ini
+
 # Portuguese -PT 'pt-pt' 0x00000816
 setxkbmap -model pc104 -layout pt
 ./xrdp-genkeymap ../instfiles/km-00000816.ini
