@@ -23,6 +23,9 @@
 
 #include "xrdp_rail.h"
 
+struct list;
+struct monitor_info;
+
 /* struct xrdp_client_info moved to xrdp_client_info.h */
 
 struct xrdp_brush
@@ -86,6 +89,9 @@ struct xrdp_drdynvc_procs
     int (*data)(intptr_t id, int chan_id, char *data, int bytes);
 };
 
+/* Defined in xrdp_client_info.h */
+struct display_size_description;
+
 /***
  * Initialise the XRDP library
  *
@@ -117,7 +123,8 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
                     int bpp, char *data, int x, int y, int cx, int cy);
 int
 libxrdp_send_pointer(struct xrdp_session *session, int cache_idx,
-                     char *data, char *mask, int x, int y, int bpp);
+                     char *data, char *mask, int x, int y, int bpp,
+                     int width, int height);
 int
 libxrdp_set_pointer(struct xrdp_session *session, int cache_idx);
 int
@@ -189,8 +196,7 @@ libxrdp_orders_send_font(struct xrdp_session *session,
                          struct xrdp_font_char *font_char,
                          int font_index, int char_index);
 int
-libxrdp_reset(struct xrdp_session *session,
-              int width, int height, int bpp);
+libxrdp_reset(struct xrdp_session *session);
 int
 libxrdp_orders_send_raw_bitmap2(struct xrdp_session *session,
                                 int width, int height, int bpp, char *data,
@@ -235,6 +241,8 @@ int
 libxrdp_disable_channel(struct xrdp_session *session, int channel_id,
                         int is_disabled);
 int
+libxrdp_drdynvc_start(struct xrdp_session *session);
+int
 libxrdp_drdynvc_open(struct xrdp_session *session, const char *name,
                      int flags, struct xrdp_drdynvc_procs *procs,
                      int *chan_id);
@@ -303,5 +311,58 @@ libxrdp_fastpath_send_frame_marker(struct xrdp_session *session,
 int EXPORT_CC
 libxrdp_send_session_info(struct xrdp_session *session, const char *data,
                           int data_bytes);
+int EXPORT_CC
+libxrdp_planar_compress(char *in_data, int width, int height,
+                        struct stream *s, int bpp, int byte_limit,
+                        int start_line, struct stream *temp_s,
+                        int e, int flags);
+/**
+ * Processes a stream that is based on either
+ *  2.2.1.3.6 Client Monitor Data (TS_UD_CS_MONITOR) or 2.2.2.2 DISPLAYCONTROL_MONITOR_LAYOUT_PDU
+ *  and then stores the processed monitor data into the description parameter.
+ * @param s
+ *      The stream to process.
+ * @param description
+ *      Must be pre-allocated. Monitor data is filled in as part of processing the stream.
+ * @param full_parameters
+ *      0 if the monitor stream is from 2.2.1.3.6 Client Monitor Data (TS_UD_CS_MONITOR)
+ *      1 if the monitor stream is from 2.2.2.2 DISPLAYCONTROL_MONITOR_LAYOUT_PDU
+ * @return 0 if the data is processed, non-zero if there is an error.
+ */
+int EXPORT_CC
+libxrdp_process_monitor_stream(struct stream *s, struct display_size_description *description,
+                               int full_parameters);
+
+/**
+ * Processes a stream that is based on [MS-RDPBCGR] 2.2.1.3.9 Client
+ * Monitor Extended Data (TS_UD_CS_MONITOR_EX)
+ *
+ * Data is stored in a struct which has already been read by
+ * libxrdp_process_monitor_stream() withj full_parameters set to 0.
+ *
+ * @param s Stream to read data from. Stream is read up to monitorAttributeSize
+ * @param description Result of reading TS_UD_CS_MONITOR PDU
+ * @return 0 if the data is processed, non-zero if there is an error.
+ */
+int EXPORT_CC
+libxrdp_process_monitor_ex_stream(struct stream *s,
+                                  struct display_size_description *description);
+
+/**
+ * Convert a list of monitors into a full description
+ *
+ * Monitor data is sanitised during the conversion
+ *
+ * @param num_monitor Monitor count (> 0)
+ * @param monitors List of monitors
+ * @param[out] description Display size description
+ *
+ * @return 0 if the data is processed, non-zero if there is an error.
+ */
+int
+libxrdp_init_display_size_description(
+    unsigned int num_monitor,
+    const struct monitor_info *monitors,
+    struct display_size_description *description);
 
 #endif
