@@ -18,6 +18,9 @@
  * media center
  */
 
+#ifndef MC_H
+#define MC_H
+
 /* include other h files */
 #include "arch.h"
 #include "parse.h"
@@ -28,6 +31,9 @@
 
 struct source_info;
 
+/* Defined in xrdp_client_info.h */
+struct monitor_info;
+
 struct mod
 {
     int size; /* size of this struct */
@@ -57,7 +63,7 @@ struct mod
     int (*server_set_cursor)(struct mod *v, int x, int y, char *data, char *mask);
     int (*server_palette)(struct mod *v, int *palette);
     int (*server_msg)(struct mod *v, const char *msg, int code);
-    int (*server_is_term)(struct mod *v);
+    int (*server_is_term)(void);
     int (*server_set_clip)(struct mod *v, int x, int y, int cx, int cy);
     int (*server_reset_clip)(struct mod *v);
     int (*server_set_fgcolor)(struct mod *v, int fgcolor);
@@ -78,7 +84,10 @@ struct mod
                             int box_left, int box_top,
                             int box_right, int box_bottom,
                             int x, int y, char *data, int data_len);
-    int (*server_reset)(struct mod *v, int width, int height, int bpp);
+    int (*client_monitor_resize)(struct mod *v, int width, int height,
+                                 int num_monitors,
+                                 const struct monitor_info *monitors);
+    int (*server_monitor_resize_done)(struct mod *v);
     int (*server_get_channel_count)(struct mod *v);
     int (*server_query_channel)(struct mod *v, int index,
                                 char *channel_name,
@@ -89,7 +98,7 @@ struct mod
                                   int total_data_len, int flags);
     int (*server_bell_trigger)(struct mod *v);
     int (*server_chansrv_in_use)(struct mod *v);
-    tintptr server_dumby[100 - 27]; /* align, 100 minus the number of server
+    tintptr server_dumby[100 - 28]; /* align, 100 minus the number of server
                                      functions above */
     /* common */
     tintptr handle; /* pointer to self as long */
@@ -102,3 +111,5 @@ struct mod
     int height;
     int bpp;
 };
+
+#endif // MC_H
