@@ -24,15 +24,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <sys/types.h>
+#include <errno.h>
+#include <poll.h>
 #include <X11/Xlib.h>
+#include <sys/select.h>
 
 Display *g_display = 0;
 int g_x_socket = 0;
 
 int main(int argc, char **argv)
 {
-    fd_set rfds;
     int i1;
     XEvent xevent;
 
@@ -48,9 +49,15 @@ int main(int argc, char **argv)
 
     while (1)
     {
-        FD_ZERO(&rfds);
-        FD_SET(g_x_socket, &rfds);
-        i1 = select(g_x_socket + 1, &rfds, 0, 0, 0);
+        struct pollfd pollfd;
+        pollfd.fd = g_x_socket;
+        pollfd.events = POLLIN;
+        pollfd.revents = 0;
+        do
+        {
+            i1 = poll(&pollfd, 1, -1);
+        }
+        while (i1 < 0 && errno == EINTR);
 
         if (i1 < 0)
         {
