@@ -36,10 +36,13 @@
 
 /* Code values used in 'xrdp_mm->code=' settings */
 #define XVNC_SESSION_CODE 0
+#define XVNC_UDS_SESSION_CODE 1
 #define XORG_SESSION_CODE 20
 
 /* To check whether touch events has been implemented on session type 'mm' */
-#define XRDP_MM_IMPLEMENTS_TOUCH(mm) ((mm)->code != XVNC_SESSION_CODE)
+#define XRDP_MM_IMPLEMENTS_TOUCH(mm) \
+    (((mm)->code != XVNC_SESSION_CODE) && \
+     ((mm)->code != XVNC_UDS_SESSION_CODE))
 
 struct source_info;
 struct list16;
@@ -441,6 +444,10 @@ struct xrdp_mm
     enum xrdp_egfx_flags egfx_flags;
     int gfx_delay_autologin;
     int mod_uses_wm_screen_for_gfx;
+    /* Whether a working h.264 library is loaded.
+     * We check this at run-time, so that we can fall-back to GFX if
+     * the H.264 library is installed incorrectly */
+    int libh264_loaded;  /* != 0 => H.264 can be used */
     /* Resize on-the-fly control */
     struct display_control_monitor_layout_data *resize_data;
     struct list *resize_queue;
